/* $Id: pcadump.c,v 1.8 1998/01/16 21:31:46 ud3 Exp $ */

#include <linux/atm.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "pca200e.h"

#define DUMP_DEVICE	0x00000001
#define DUMP_CPQUEUE	0x00000002
#define DUMP_MONITOR	0x00000004
#define DUMP_STATS	0x00000008


/* media type definitions */
#define MT_ESP_CAT5_UTP      	0x6
#define MT_ESP_MMODE_OC3_ST    	0x16
#define MT_ESP_MMODE_OC3_SC    	0x26

/* prototypes */
int usage(char*, int);
unsigned int dmaswap(unsigned int in);
void showdump(struct pca200e_dump *, int, unsigned long);

int
usage(char* name, int rv)
{
  printf("shows state and statistics of PCA-200EPC\n\n"
	 "usage: %s [-d] [-c] [-m] [-s] [-a] [-h] <itf> [<itf> ...]\n"
	 "  <itf>\t\tatm interface number\n"
	 "  -d\t\tshow device info (default)\n"
	 "  -c\t\tshow cpqueue info\n"
	 "  -m\t\tshow monitor info\n"
	 "  -s\t\tshow device statistics\n"
	 "  -a\t\tlike -d -c -m -s\n"
	 "  -h\t\tshow this help text\n\n",
	 name);
  exit(rv);
};

unsigned int
dmaswap(unsigned int in)
{
#ifndef __alpha__
  asm("bswap %%eax\n"
      : "=a" (in)
      : "a" (in));
#endif
  return in;
};

void
showdump(struct pca200e_dump* dump, int itf, unsigned long flags)
{
  printf("FORE Systems PCA-200E dump for itf %d\n", itf);
  if (flags & DUMP_DEVICE)
    {
      printf("\n"
	     " device:\n\n");
      printf("   internal name:\t%s\n", dump->dev.name);
      printf("   PCI bus number:\t%d\n", dump->dev.bus);
      printf("   device function:\t0x%x\n", dump->dev.dev_fn);
      printf("   interrupt line:\t%d\n", dump->dev.irq);
      printf("   physical rambase:\t0x%08x\n", (unsigned) dump->dev.phys_rambase);
      printf("   virtual rambase:\t0x%08x\n", (unsigned) dump->dev.virt_rambase);
      printf("   HW address (ESI):\t%02x:%02x:%02x:%02x:%02x:%02x\n",
	     dump->dev.mac[0], dump->dev.mac[1], dump->dev.mac[2],
	     dump->dev.mac[3], dump->dev.mac[4], dump->dev.mac[5]);
      printf("   Serial No:\t\t%d\n", dump->dev.mac[4] * 256 +
				      dump->dev.mac[5]);
      printf("   active small buffs:\t%d\n", dump->dev.smlb1_count);
      printf("   active large buffs:\t%d\n", dump->dev.lrgb1_count);
      printf("   card init state:\t%d\n", dump->dev.cardstate);


    };
  if (flags & DUMP_CPQUEUE)
    {
      printf("\n\n"
	     " cpqueue:\n\n");
      printf("   heartbeat state:\t");
      if (0xdead != (dump->cpqueue.heartbeat >> 16)) 
	printf("0x%08x\n", dump->cpqueue.heartbeat);
      else
	printf("*** FATAL ERROR %04x ***\n",
	       dump->cpqueue.heartbeat & 0x0000FFFF);
      printf("   firmware release:\t%d.%d.%d\n",
	     dump->cpqueue.firmware_release >> 16,
	     dump->cpqueue.firmware_release << 16 >> 24,
	     dump->cpqueue.firmware_release << 24 >> 24);
      printf("   monitor release:\t%d.%d\n",
	     dump->cpqueue.mon960_release >> 16,
	     dump->cpqueue.mon960_release << 16 >> 16);
      printf("   media type:\t\t%s\n",
	     dump->cpqueue.media_type == MT_ESP_CAT5_UTP ? "Unshielded twisted pair, 155 Mbps OC3" :
	     dump->cpqueue.media_type == MT_ESP_MMODE_OC3_SC ? "MultiMode 155 Mbps OC3 SC" :
	     dump->cpqueue.media_type == MT_ESP_MMODE_OC3_ST ? "MultiMode 155 Mbps OC3 ST" :
	     "unknown");
      // printf("   OC3 revision: \t%d\n", dump->cpqueue.oc3_revision);
    };
  if (flags & DUMP_MONITOR)
    {
      printf("\n\n"
	     " monitor:\n\n");
      printf("   boot status word:\t0x%08x\n", dump->monitor.bstat);
      printf("   version number:\t%d\n", dump->monitor.version);
    };
  if (flags & DUMP_STATS)
    {
      stats200_t* stat = &dump->stats;
      printf("\n\n"
	     " device statistics:%s\n",
	     (0xdead != (dump->cpqueue.heartbeat >> 16)) ?
	     "" :
	     " (may be incorrect (->heartbeat))");
      printf("\n"
	     "   PHY_4b5b:\n"
	     "     crc_header_errors:\t\t%10u\n"
	     "     framing_errors:\t\t%10u\n",
	     dmaswap(stat->phy4b5b.crc_header_errors),
	     dmaswap(stat->phy4b5b.framing_errors));
      
      printf("\n"
	     "   PHY_OC3:\n"
	     "     section_bip8_errors:\t%10u\n"
	     "     path_bip8_errors:\t\t%10u\n"
	     "     line_bip24_errors:\t\t%10u\n"
             "     line_febe_errors:\t\t%10u\n"
	     "     path_febe_errors:\t\t%10u\n"
	     "     corr_hcs_errors:\t\t%10u\n"
	     "     ucorr_hcs_errors:\t\t%10u\n",
	     dmaswap(stat->phyoc3.section_bip8_errors),
	     dmaswap(stat->phyoc3.path_bip8_errors),
	     dmaswap(stat->phyoc3.line_bip24_errors),
	     dmaswap(stat->phyoc3.line_febe_errors),
	     dmaswap(stat->phyoc3.path_febe_errors),
	     dmaswap(stat->phyoc3.corr_hcs_errors),
	     dmaswap(stat->phyoc3.ucorr_hcs_errors));

      printf("\n"
	     "   ATM:\t\t\t\t     cells\n"
	     "     TX:\t\t\t%10u\n"
	     "     RX:\t\t\t%10u\n",
	     dmaswap(stat->atm.cells_transmitted),
	     dmaswap(stat->atm.cells_received));
      printf("     vpi out of range:\t\t%10u\n"
	     "     vpi no conn:\t\t%10u\n",
	     dmaswap(stat->atm.vpi_bad_range),
	     dmaswap(stat->atm.vpi_no_conn));
      printf("     vci out of range:\t\t%10u\n"
	     "     vci no conn:\t\t%10u\n",
	     dmaswap(stat->atm.vci_bad_range),
	     dmaswap(stat->atm.vci_no_conn));
      
      printf("\n"
	     "   AAL0:\t\t\t     cells\n"
	     "     TX:\t\t\t%10u\n"
	     "     RX:\t\t\t%10u\n"
	     "     dropped:\t\t\t%10u\n",
	     dmaswap(stat->aal0.cells_transmitted),
	     dmaswap(stat->aal0.cells_received),
	     dmaswap(stat->aal0.cells_dropped));
      
      printf("\n"
	     "   AAL4:\n"
	     "     SAR sublayer:\t\t     cells\n"
	     "       TX:\t\t\t%10u\n"
	     "       RX:\t\t\t%10u\n"
	     "       dropped:\t\t\t%10u\n"
	     "       CRC errors:\t\t%10u\n"
	     "       protocol errors:\t\t%10u\n\n"
	     "     CS  sublayer:\t\t      PDUs\n"
	     "       TX:\t\t\t%10u\n"
	     "       RX:\t\t\t%10u\n"
	     "       dropped:\t\t\t%10u\n"
	     "       protocol errors:\t\t%10u\n",
	     dmaswap(stat->aal4.cells_transmitted),
	     dmaswap(stat->aal4.cells_received),
	     dmaswap(stat->aal4.cells_dropped),
	     dmaswap(stat->aal4.cells_crc_errors),
	     dmaswap(stat->aal4.cells_protocol_errors),
	     dmaswap(stat->aal4.cspdus_transmitted),
	     dmaswap(stat->aal4.cspdus_received),
	     dmaswap(stat->aal4.cspdus_dropped),
	     dmaswap(stat->aal4.cspdus_protocol_errors));

      printf("\n"
	     "   AAL5:\n"
	     "     SAR sublayer:\t\t     cells\n"
	     "       TX:\t\t\t%10u\n"
	     "       RX:\t\t\t%10u\n"
	     "       dropped:\t\t\t%10u\n"
	     "       congestions:\t\t%10u\n\n"
	     "     CS  sublayer:\t\t      PDUs\n"
	     "       TX:\t\t\t%10u\n"
	     "       RX:\t\t\t%10u\n"
	     "       dropped:\t\t\t%10u\n"
	     "       CRC errors:\t\t%10u\n"
	     "       protocol errors:\t\t%10u\n",
	     dmaswap(stat->aal5.cells_transmitted),
	     dmaswap(stat->aal5.cells_received),
	     dmaswap(stat->aal5.cells_dropped),
	     dmaswap(stat->aal5.congestion_experienced),
	     dmaswap(stat->aal5.cspdus_transmitted),
	     dmaswap(stat->aal5.cspdus_received),
	     dmaswap(stat->aal5.cspdus_dropped),
	     dmaswap(stat->aal5.cspdus_crc_errors),
	     dmaswap(stat->aal5.cspdus_protocol_errors));

      printf("\n"
	     "   AUX:\t\t       allocation failures\n"
	     "     small b1:\t\t\t%10u\n"
	     "     large b1:\t\t\t%10u\n"
	     "     small b2:\t\t\t%10u\n"
	     "     large b2:\t\t\t%10u\n"
	     "     RX PDUs:\t\t\t%10u\n",
	     dmaswap(stat->aux200.small_b1_failed),
	     dmaswap(stat->aux200.large_b1_failed),
	     dmaswap(stat->aux200.small_b2_failed),
	     dmaswap(stat->aux200.large_b2_failed),
	     dmaswap(stat->aux200.rpd_alloc_failed));
      printf("\n"
	     "   Receive carrier:\t\t\t%s\n",stat->aux200.receive_carrier?"ON":"OFF!");
  

    };
};


int
main(int argc, char* argv[])
{
  int s;
  int rc;
  unsigned long flags; 
  struct atmif_sioc req;
  struct pca200e_dump dump;
  
  flags = DUMP_DEVICE;

  optind = 0;
  while ((rc = getopt(argc,argv,"dcmsah")) != EOF)
    switch (rc) {
    case 'd':
      flags |= DUMP_DEVICE;
      break;
    case 'c':
      flags |= DUMP_CPQUEUE;
      break;
    case 'm':
      flags |= DUMP_MONITOR;
      break;
    case 's':
      flags |= DUMP_STATS;
      break;
    case 'a':
      flags |= ~0;
      break;
    case 'h':
      usage(argv[0], 0);
    default :
      usage(argv[0], -1);
    };
  
  if (!(optind < argc))
    usage(argv[0], -1);

  while (optind < argc)
    {
      memset(&dump, 0, sizeof(struct pca200e_dump));
      req.number = atol(argv[optind++]);
      req.arg = &dump;
      req.length = sizeof(dump);
      s = socket(PF_ATMPVC,SOCK_DGRAM,0);
      if ((rc = ioctl(s, PCA200E_DUMP, &req)) < 0)
	printf("\n%s: no interface %d\n", argv[0], req.number);
      else
	showdump(&dump, req.number, flags);
    };
  
  return 0;
};


